"""cg module for cleaning databases and files"""
import logging
import shutil
from datetime import datetime
from pathlib import Path
from typing import List

import click
from dateutil.parser import parse as parse_date

from cg.apps.balsamic.api import BalsamicAPI
from cg.apps.balsamic.fastq import FastqHandler
from cg.apps.crunchy import CrunchyAPI
from cg.apps.hermes.hermes_api import HermesApi
from cg.apps.housekeeper.hk import HousekeeperAPI
from cg.apps.lims import LimsAPI
from cg.apps.scout.scout_export import ScoutExportCase
from cg.apps.scout.scoutapi import ScoutAPI
from cg.apps.tb import TrailblazerAPI
from cg.constants import EXIT_FAIL, EXIT_SUCCESS
from cg.meta.workflow.balsamic import BalsamicAnalysisAPI
from cg.meta.workflow.mip import MipAnalysisAPI
from cg.store import Store

LOG = logging.getLogger(__name__)


@click.group()
@click.pass_context
def clean(context):
    """Clean up processes"""
    context.obj["status_db"] = Store(context.obj["database"])
    context.obj["housekeeper_api"] = HousekeeperAPI(context.obj)
    context.obj["trailblazer_api"] = TrailblazerAPI(context.obj)
    context.obj["scout_api"] = ScoutAPI(context.obj)
    context.obj["crunchy_api"] = CrunchyAPI(context.obj)
    context.obj["lims_api"] = LimsAPI(context.obj)
    context.obj["hermes_api"] = HermesApi(context.obj)

    context.obj["BalsamicAnalysisAPI"] = BalsamicAnalysisAPI(
        balsamic_api=BalsamicAPI(context.obj),
        store=context.obj["status_db"],
        housekeeper_api=context.obj["housekeeper_api"],
        fastq_handler=FastqHandler(context.obj),
        lims_api=context.obj["lims_api"],
        trailblazer_api=context.obj["trailblazer_api"],
        hermes_api=context.obj["hermes_api"],
    )
    context.obj["MipAnalysisAPI"] = MipAnalysisAPI(
        db=context.obj["status_db"],
        hk_api=context.obj["housekeeper_api"],
        tb_api=context.obj["trailblazer_api"],
        scout_api=context.obj["scout_api"],
        lims_api=context.obj["lims_api"],
        script=context.obj["mip-rd-dna"]["script"],
        pipeline=context.obj["mip-rd-dna"]["pipeline"],
        conda_env=context.obj["mip-rd-dna"]["conda_env"],
        root=context.obj["mip-rd-dna"]["root"],
    )


@clean.command("balsamic-run-dir")
@click.option("-y", "--yes", is_flag=True, help="Skip confirmation")
@click.option("-d", "--dry-run", is_flag=True, help="Shows cases and files that would be cleaned")
@click.argument("case_id")
@click.pass_context
def balsamic_run_dir(context, yes, case_id, dry_run: bool = False):
    """Remove Balsamic run directory"""

    balsamic_analysis_api = context.obj["BalsamicAnalysisAPI"]
    case_object = balsamic_analysis_api.get_case_object(case_id)
    if not case_object:
        LOG.warning(f"{case_id} not found!")
        raise click.Abort()

    analysis_obj = case_object.analyses[0] if case_object.analyses else None
    if not analysis_obj:
        LOG.error(f"{case_id}: analysis not found")
        raise click.Abort()

    analysis_path = Path(balsamic_analysis_api.get_case_path(case_id))

    if dry_run:
        LOG.info(f"Would have deleted: {analysis_path}")
        return EXIT_SUCCESS

    if yes or click.confirm(f"Are you sure you want to remove all files in {analysis_path}?"):
        if not analysis_path.exists():
            LOG.warning(f"Could not find path: {analysis_path}")
            return EXIT_FAIL
        if analysis_path.is_symlink():
            LOG.warning(
                f"Will not automatically delete symlink: {analysis_path}, delete it manually",
            )
            return EXIT_FAIL

        shutil.rmtree(analysis_path, ignore_errors=True)
        LOG.info(f"Cleaned {analysis_path}")
        analysis_obj.cleaned_at = datetime.now()
        balsamic_analysis_api.store.commit()


@clean.command("mip-run-dir")
@click.option("-y", "--yes", is_flag=True, help="Skip confirmation")
@click.option("-d", "--dry-run", is_flag=True, help="Shows cases and files that would be cleaned")
@click.argument("case_id")
@click.pass_context
def mip_run_dir(context, yes, case_id, dry_run: bool = False):
    """Remove MIP run directory"""

    mip_analysis_api = context.obj["MipAnalysisAPI"]
    case_obj = mip_analysis_api.get_case_object(case_id)

    if case_obj is None:
        LOG.error("%s: family not found", case_id)
        raise click.Abort()

    analysis_path = mip_analysis_api.get_case_output_path(case_id=case_id)
    if not analysis_path.exists():
        LOG.error(f"No analysis directory found for {case_id}")
        raise click.Abort()

    if dry_run:
        LOG.info(f"Cleaning case {case_id} : Would have deleted contents of {analysis_path}")
        return

    if yes or click.confirm(f"Are you sure you want to remove {case_id}?"):
        shutil.rmtree(analysis_path, ignore_errors=True)
        LOG.info(f"Cleaning case {case_id} : Deleted contents of {analysis_path}")
        mip_analysis_api.mark_analyses_deleted(case_id)
        for analysis_obj in case_obj.analyses:
            analysis_obj.cleaned_at = analysis_obj.cleaned_at or datetime.now()
        mip_analysis_api.db.commit()


@clean.command("hk-alignment-files")
@click.argument("bundle")
@click.option("-y", "--yes", is_flag=True, help="Skip confirmation")
@click.option("-d", "--dry-run", is_flag=True, help="Show files that would be cleaned")
@click.pass_context
def hk_alignment_files(context, bundle, yes: bool = False, dry_run: bool = False):
    """Clean up alignment files in Housekeeper bundle"""
    if bundle is None:
        LOG.info("Please select a bundle")
        raise click.Abort
    files = []
    for tag in ["bam", "bai", "bam-index", "cram", "crai", "cram-index"]:
        files.extend(context.obj["housekeeper_api"].get_files(bundle=bundle, tags=[tag]))
    for file_obj in files:
        if file_obj.is_included:
            question = f"{bundle}: remove file from file system and database: {file_obj.full_path}"
        else:
            question = f"{bundle}: remove file from database: {file_obj.full_path}"

        if yes or click.confirm(question):
            file_name = file_obj.full_path
            if file_obj.is_included and Path(file_name).exists():
                if not dry_run:
                    Path(file_name).unlink()

            if not dry_run:
                file_obj.delete()
                context.obj["housekeeper_api"].commit()
                click.echo(f"{file_name} deleted")


@clean.command("scout-finished-cases")
@click.option(
    "--days-old",
    type=int,
    default=300,
    help="Clean alignment files with analysis dates older then given number of days",
)
@click.option("-y", "--yes", is_flag=True, help="Skip confirmation")
@click.option("-d", "--dry-run", is_flag=True, help="Shows cases and files that would be cleaned")
@click.pass_context
def scout_finished_cases(context, days_old: int, yes: bool = False, dry_run: bool = False):
    """Clean up of solved and archived scout cases"""
    bundles = []
    for status in ["archived", "solved"]:
        cases: List[ScoutExportCase] = context.obj["scout_api"].get_cases(
            status=status, reruns=False
        )
        cases_added = 0
        for case in cases:
            x_days_ago = datetime.now() - case.analysis_date
            if x_days_ago.days > days_old:
                bundles.append(case.id)
                cases_added += 1
        LOG.info("%s cases marked for bam removal", cases_added)

    for bundle in bundles:
        context.invoke(hk_alignment_files, bundle=bundle, yes=yes, dry_run=dry_run)


@clean.command("hk-past-files")
@click.option("-c", "--case-id", type=str)
@click.option("-t", "--tags", multiple=True)
@click.option("-y", "--yes", is_flag=True, help="skip checks")
@click.option("-d", "--dry-run", is_flag=True, help="Shows cases and files that would be cleaned")
@click.pass_context
def hk_past_files(context, case_id, tags, yes, dry_run):
    """ Remove files found in older housekeeper bundles """
    if case_id:
        cases = [context.obj["status_db"].family(case_id)]
    else:
        cases = context.obj["status_db"].families()
    for case in cases:
        case_id = case.internal_id
        last_version = context.obj["housekeeper_api"].last_version(bundle=case_id)
        if not last_version:
            continue
        last_version_file_paths = [
            Path(hk_file.full_path)
            for hk_file in context.obj["housekeeper_api"].get_files(
                bundle=case_id, tags=None, version=last_version.id
            )
        ]
        LOG.info("Searching %s bundle for outdated files", case_id)
        hk_files = []
        for tag in tags:
            hk_files.extend(context.obj["housekeeper_api"].get_files(bundle=case_id, tags=[tag]))
        for hk_file in hk_files:
            file_path = Path(hk_file.full_path)
            if file_path in last_version_file_paths:
                continue
            LOG.info("Will remove %s", file_path)
            if yes or click.confirm("Do you want to remove this file?"):
                if not dry_run:
                    hk_file.delete()
                    context.obj["housekeeper_api"].commit()
                    if file_path.exists():
                        file_path.unlink()
                    LOG.info("File removed")


@clean.command("balsamic-past-run-dirs")
@click.option("-y", "--yes", is_flag=True, help="Skip confirmation")
@click.option("-d", "--dry-run", is_flag=True, help="Shows cases and files that would be cleaned")
@click.argument("before_str")
@click.pass_context
def balsamic_past_run_dirs(context, before_str: str, yes: bool = False, dry_run: bool = False):
    """Clean up of "old" Balsamic case run dirs"""

    exit_code = EXIT_SUCCESS
    before = parse_date(before_str)
    balsamic_analysis_api = context.obj["BalsamicAnalysisAPI"]
    possible_cleanups = balsamic_analysis_api.get_analyses_to_clean(before_date=before)
    LOG.info(f"Cleaning {len(possible_cleanups)} analyses created before {before}")

    for analysis in possible_cleanups:
        case_id = analysis.family.internal_id
        LOG.info(f"{case_id} - {balsamic_analysis_api.store.family(case_id).action}")
        try:
            LOG.info(f"Cleaning Balsamic output for {case_id}")
            context.invoke(balsamic_run_dir, yes=yes, case_id=case_id, dry_run=dry_run)
        except Exception as e:
            LOG.error(f"Failed to clean directories for case {case_id} - {e.__class__}")
            exit_code = EXIT_FAIL
    if exit_code:
        raise click.Abort

    LOG.info("Done cleaning Balsamic output")


@clean.command("mip-past-run-dirs")
@click.option("-y", "--yes", is_flag=True, help="Skip confirmation")
@click.option("-d", "--dry-run", is_flag=True, help="Shows cases and files that would be cleaned")
@click.argument("before_str")
@click.pass_context
def mip_past_run_dirs(
    context: click.Context, before_str: str, yes: bool = False, dry_run: bool = False
):
    """Clean up of "old" MIP case run dirs"""
    exit_code = EXIT_SUCCESS
    mip_analysis_api = context.obj["MipAnalysisAPI"]
    before = parse_date(before_str)
    old_analyses = mip_analysis_api.get_analyses_to_clean(before=before)
    LOG.info(f"Cleaning {len(old_analyses)} analyses created before {before}")
    for status_analysis in old_analyses:
        case_id = status_analysis.family.internal_id
        try:
            LOG.info(f"Cleaning MIP output for {case_id}")
            context.invoke(
                mip_run_dir,
                yes=yes,
                case_id=case_id,
                dry_run=dry_run,
            )
        except Exception as e:
            LOG.error(
                f"Failed to clean directories for case {status_analysis.family.internal_id} - {e.__class__}"
            )
            exit_code = EXIT_FAIL
            continue
    if exit_code:
        raise click.Abort

    LOG.info("Done cleaning MIP output")



"""Tests for the cli workflows get links function"""

import click
import pytest
from cg.cli.workflow.get_links import get_links


def test_get_links_case_id(analysis_store):
    """Test to get links for a case id in a populated store with linked samples"""
    # GIVEN a populated store with linked samples
    case_obj = analysis_store.Family.query.first()
    assert case_obj.links

    # WHEN fetching the links for the case
    link_objs = get_links(store=analysis_store, case_id=case_obj.internal_id)

    # THEN assert that the correct links where found
    assert link_objs == case_obj.links


def test_get_links_non_existing_case_id(analysis_store, caplog):
    """Test to get links for a case id in a populated store with linked samples"""
    # GIVEN a populated store with linked samples and a non existing case id
    case_id = "hello"
    case_obj = analysis_store.family(case_id)
    assert not case_obj

    # WHEN fetching the links for the case
    with pytest.raises(click.Abort):
        # THEN assert that it was aborted
        get_links(store=analysis_store, case_id=case_id)
        # THEN assert it communicates that the case_id could not be found
        assert f"could not find case {case_id}" in caplog.text


def test_get_links_case_id_no_links(base_store, helpers, workflow_case_id, caplog):
    """Test to get links for a case id in a populated store where samples are NOT linked"""
    # GIVEN a populated store with unlinked samples
    helpers.add_case(base_store, workflow_case_id)
    helpers.add_sample(base_store, "dna_sample", is_rna=False)

    case_obj = base_store.Family.query.first()
    case_id = case_obj.internal_id
    assert case_obj
    assert not case_obj.links

    # WHEN fetching the links for the case
    with pytest.raises(click.Abort):
        # THEN assert that it was aborted
        get_links(store=base_store, case_id=case_id)
        # THEN assert it communicates that the case_id could not be found
        assert f"Could not find links for case {case_id}" in caplog.text


def test_get_links_sample_id_and_case_id(analysis_store):
    """Test to get links for a sample id in a populated store with linked samples"""
    # GIVEN a populated store with linked samples
    sample_obj = analysis_store.Sample.query.first()
    assert sample_obj
    sample_id = sample_obj.internal_id

    case_obj = analysis_store.Family.query.first()
    assert case_obj
    case_id = case_obj.internal_id

    link_obj = analysis_store.link(family_id=case_id, sample_id=sample_id)
    assert link_obj

    # WHEN fetching the links for the sample in specific case
    link_objs = get_links(store=analysis_store, case_id=case_id, sample_id=sample_id)
    # THEN assert that the correct links where found
    assert link_objs == [link_obj]


def test_get_links_case_id_and_sample_id_no_links(base_store, helpers, workflow_case_id, caplog):
    """Test to get links for a case id in a populated store where samples are NOT linked"""
    # GIVEN a populated store with unlinked samples
    helpers.add_case(base_store, workflow_case_id)
    helpers.add_sample(base_store, "dna_sample", is_rna=False)

    case_obj = base_store.Family.query.first()
    assert case_obj
    case_id = case_obj.internal_id
    assert not case_obj.links

    sample_obj = base_store.Sample.query.first()
    assert sample_obj
    sample_id = sample_obj.internal_id
    assert not sample_obj.links

    # WHEN fetching the links for the case
    with pytest.raises(click.Abort):
        # THEN assert that it was aborted
        get_links(store=base_store, case_id=case_id, sample_id=sample_id)
        # THEN assert it communicates that the case_id could not be found
        assert f"Could not find link for case {case_id} and sample {sample_id}" in caplog.text


def test_get_links_sample_id(analysis_store):
    """Test to get links for a sample id in a populated store with linked samples"""
    # GIVEN a populated store with linked samples
    sample_obj = analysis_store.Sample.query.first()
    assert sample_obj.links

    # WHEN fetching the links for the sample
    link_objs = get_links(store=analysis_store, sample_id=sample_obj.internal_id)
    # THEN assert that the correct links where found
    assert link_objs == sample_obj.links


def test_get_links_non_existing_sample_id(analysis_store, caplog):
    """Test to get links for a sample_id that does not exist"""
    # GIVEN a populated store with linked samples
    non_existing = "hello"
    sample_obj = analysis_store.sample(non_existing)
    assert not sample_obj

    # WHEN fetching the links for the sample
    with pytest.raises(click.Abort):
        # THEN assert that an exception is raised
        get_links(store=analysis_store, sample_id=non_existing)
        # THEN assert it communicates that sample_id did not exist
        assert "Could not find sample" in caplog.text


def test_get_links_sample_id_no_links(base_store, helpers, caplog):
    """Test to get links for a sample in a populated store where samples are NOT linked"""
    # GIVEN a populated store with unlinked samples
    helpers.add_sample(base_store, "dna_sample", is_rna=False)

    sample_obj = base_store.Sample.query.first()
    assert sample_obj
    sample_id = sample_obj.internal_id
    assert not sample_obj.links

    # WHEN fetching the links for the case
    with pytest.raises(click.Abort):
        # THEN assert that it was aborted
        get_links(store=base_store, sample_id=sample_id)
        # THEN assert it communicates that the case_id could not be found
        assert f"Could not find any links" in caplog.text


def test_get_links_no_case_id_no_sample_id(analysis_store, caplog):
    """Test to get links for a case id in a populated store with linked samples"""
    # GIVEN a populated store with linked samples

    # WHEN fetching the links for the case without giving sample_id or case_is
    with pytest.raises(click.Abort):
        # THEN assert that an exception is raised
        get_links(store=analysis_store)
        # THEN assert it communicates that sample_id and/or case_id must be used
        assert "provide case and/or sample" in caplog.text



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import GenericRepr, Snapshot


snapshots = Snapshot()

snapshots["test_build_dna_bundle 1"] = {
    "created": GenericRepr("datetime.datetime(2020, 3, 1, 0, 0)"),
    "files": {"path": "mock_path"},
    "name": "case_id",
    "pipeline_version": "v8.2.2",
}

snapshots["test_build_rna_bundle 1"] = {
    "created": GenericRepr("datetime.datetime(2020, 3, 1, 0, 0)"),
    "files": {"path": "mock_path"},
    "name": "case_id",
    "pipeline_version": "v8.2.2",
}

snapshots["test_parse_files_dna 1"] = [
    {
        "archive": False,
        "deliverables_tag_map": ("gatk_baserecalibration",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/gatk_baserecalibration/sample_id_lanes_5555_sorted_md_brecal.cram",
        "tags": ["cram", "mip-dna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("gatk_baserecalibration",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/gatk_baserecalibration/sample_id_lanes_5555_sorted_md_brecal.cram.crai",
        "tags": ["cram-index", "mip-dna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chanjo_sexcheck",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chanjo_sexcheck/sample_id_lanes_5555_sorted_md_brecal_sex.tsv",
        "tags": ["chanjo", "mip-dna", "sample_id", "sex-check"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("samtools_subsample_mt",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/samtools_subsample_mt/sample_id_lanes_5555_sorted_md_brecal_subsample_MT.bam",
        "tags": ["bam-mt", "mip-dna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("samtools_subsample_mt",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/samtools_subsample_mt/sample_id_lanes_5555_sorted_md_brecal_subsample_MT.bam.bai",
        "tags": ["bam-mt-index", "mip-dna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("smncopynumbercaller",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/smncopynumbercaller/sample_id_lanes_5555_sorted_md_brecal_smn.tsv",
        "tags": ["mip-dna", "sample_id", "smn-calling", "smncopynumbercaller"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("expansionhunter", "sv_str"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/expansionhunter/case_id_exphun.vcf.gz",
        "tags": ["case_id", "mip-dna", "vcf-str"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("expansionhunter", "sv_str"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/expansionhunter/case_id_exphun.vcf.gz.csi",
        "tags": ["case_id", "mip-dna", "vcf-str-index"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("sv_combinevariantcallsets",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/sv_combinevariantcallsets/case_id_comb.bcf",
        "tags": ["case_id", "mip-dna", "sv-bcf"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("sv_combinevariantcallsets",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/sv_combinevariantcallsets/case_id_comb.bcf.csi",
        "tags": ["case_id", "mip-dna", "sv-bcf-index"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("sv_reformat", "research"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/sv_reformat/case_id_comb_ann_vep_parsed_ranked.vcf.gz",
        "tags": ["case_id", "mip-dna", "vcf-sv-research"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("sv_reformat", "research"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/sv_reformat/case_id_comb_ann_vep_parsed_ranked.vcf.gz.csi",
        "tags": ["case_id", "mip-dna", "vcf-sv-research-index"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("sv_reformat", "clinical"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/sv_reformat/case_id_comb_ann_vep_parsed_ranked.selected.vcf.gz",
        "tags": ["case_id", "mip-dna", "vcf-sv-clinical"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("sv_reformat", "clinical"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/sv_reformat/case_id_comb_ann_vep_parsed_ranked.selected.vcf.gz.csi",
        "tags": ["case_id", "mip-dna", "vcf-sv-clinical-index"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("vcf2cytosure_ar",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/vcf2cytosure_ar/case_id_cyto.sample_id.cgh",
        "tags": ["mip-dna", "sample_id", "vcf2cytosure"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("peddy_ar", "peddy"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/peddy_ar/case_id_gatkcomb.peddy.ped",
        "tags": ["case_id", "mip-dna", "ped", "peddy"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("peddy_ar", "ped_check"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/peddy_ar/case_id_gatkcomb.ped_check.csv",
        "tags": ["case_id", "mip-dna", "ped-check", "peddy"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("peddy_ar", "sex_check"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/peddy_ar/case_id_gatkcomb.sex_check.csv",
        "tags": ["case_id", "mip-dna", "peddy", "sex-check"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("endvariantannotationblock", "research"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/endvariantannotationblock/case_id_gatkcomb_rhocall_vt_af_frqf_cadd_vep_parsed_ranked.vcf.gz",
        "tags": ["case_id", "mip-dna", "vcf-snv-research"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("endvariantannotationblock", "research"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/endvariantannotationblock/case_id_gatkcomb_rhocall_vt_af_frqf_cadd_vep_parsed_ranked.vcf.gz.tbi",
        "tags": ["case_id", "mip-dna", "vcf-snv-research-index"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("endvariantannotationblock", "clinical"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/endvariantannotationblock/case_id_gatkcomb_rhocall_vt_af_frqf_cadd_vep_parsed_ranked.selected.vcf.gz",
        "tags": ["case_id", "mip-dna", "vcf-snv-clinical"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("endvariantannotationblock", "clinical"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/endvariantannotationblock/case_id_gatkcomb_rhocall_vt_af_frqf_cadd_vep_parsed_ranked.selected.vcf.gz.tbi",
        "tags": ["case_id", "mip-dna", "vcf-snv-clinical-index"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("qccollect_ar",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/qccollect_ar/case_id_qc_metrics.yaml",
        "tags": ["case_id", "mip-dna", "qcmetrics"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("version_collect_ar",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/version_collect_ar/case_id_vcol.yaml",
        "tags": ["case_id", "exe-ver", "mip-dna"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("multiqc_ar", "html"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/multiqc_ar/multiqc_report.html",
        "tags": ["case_id", "mip-dna", "multiqc-html"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("multiqc_ar", "json"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/multiqc_ar/multiqc_data/multiqc_data.json",
        "tags": ["case_id", "mip-dna", "multiqc-json"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("gatk_combinevariantcallsets",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/gatk_combinevariantcallsets/case_id_gatkcomb.bcf",
        "tags": ["case_id", "mip-dna", "snv-bcf", "snv-gbcf"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("gatk_combinevariantcallsets",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/gatk_combinevariantcallsets/case_id_gatkcomb.bcf.csi",
        "tags": ["case_id", "gbcf-index", "mip-dna"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("sambamba_depth", "coverage"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/sambamba_depth/sample_id_lanes_5555_sorted_md_brecal_coverage.bed",
        "tags": ["coverage", "mip-dna", "sambamba-depth", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_1.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_2.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_3.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_4.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_5.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_6.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_7.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_8.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_9.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_10.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_11.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_12.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_13.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_14.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_15.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_16.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_17.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_18.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_19.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_20.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_21.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_22.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_X.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_cov", "tcov"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_cov/sample_id_lanes_1_sorted_md_brecal_tcov_Y.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "tcov"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_7.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_3.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_9.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_6.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_21.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_15.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_Y.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_10.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_X.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_17.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_3.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_16.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_13.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_17.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_1.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_9.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_10.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_Y.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_16.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_2.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_22.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_4.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_19.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_15.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_5.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_8.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_7.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_11.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_21.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_13.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_12.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_18.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_2.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_X.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_6.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_19.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_20.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_14.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_12.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_11.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_8.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_5.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_18.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_4.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_22.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.regions_14.png",
        "tags": ["chromograph", "mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_1.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("chromograph_upd", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/chromograph_upd/sample_id_gatkcomb_rhocall_vt_af_upd.sites_20.png",
        "tags": ["chromograph", "mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("rhocall_viz",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/rhocall_viz/case_id_gatkcomb_rhocall_vt_af_rhoviz.sample_id.bw",
        "tags": ["mip-dna", "rhocall-viz", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("upd_ar", "sites"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/upd_ar/sample_id_gatkcomb_rhocall_vt_af_upd.sites.bb",
        "tags": ["mip-dna", "sample_id", "sites", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("upd_ar", "regions"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/upd_ar/sample_id_gatkcomb_rhocall_vt_af_upd.regions.bb",
        "tags": ["mip-dna", "regions", "sample_id", "upd"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "log"),
        "path": "/home/proj/production/rare-disease/cases/flyingsnipe/analysis/mip_log/YYYY-MM-DD/mip_YYYY-MM-DDTHH:MM:SS.log",
        "tags": ["case_id", "mip-dna", "mip-log"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "pedigree"),
        "path": "/path/to/rare-disease/cases/case_id/pedigree.yaml",
        "tags": ["case_id", "mip-dna", "pedigree-yaml"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "config"),
        "path": "/path/to/mip/config/mip_version-dna.yaml",
        "tags": ["case_id", "config", "mip-analyse", "mip-dna"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "pedigree_fam"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/case_id.fam",
        "tags": ["case_id", "mip-dna", "pedigree"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "references_info"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/reference_info.yaml",
        "tags": ["case_id", "mip-analyse", "mip-dna", "reference-info"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "sample_info"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id_qc_sample_info.yaml",
        "tags": ["case_id", "mip-dna", "sampleinfo"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "config_analysis"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id_config.yaml",
        "tags": ["case_id", "mip-config", "mip-dna"],
    },
]

snapshots["test_parse_files_rna 1"] = [
    {
        "archive": False,
        "deliverables_tag_map": ("salmon_quant",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/salmon_quant/quant.sf",
        "tags": ["mip-rna", "salmon-quant", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("star_fusion",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/star_fusion/star-fusion.fusion_predictions.abridged.tsv",
        "tags": ["mip-rna", "sample_id", "star-fusion"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("arriba_ar", "arriba_report"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/arriba_ar/sample_id_lanes_1234_trim_arriba.pdf",
        "tags": ["arriba-ar", "arriba-report", "mip-rna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("arriba_ar", "arriba_ar"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/arriba_ar/sample_id_lanes_1234_trim_arriba.tsv",
        "tags": ["arriba-ar", "mip-rna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("stringtie_ar",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/stringtie_ar/sample_id_lanes_1234_trim_star_sorted_merged_strg.gtf",
        "tags": ["mip-rna", "sample_id", "stringtie-ar"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("gffcompare_ar",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/gffcompare_ar/sample_id_lanes_1234_trim_star_sorted_merged_strg_gffcmp.gtf",
        "tags": ["gffcompare-ar", "mip-rna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("markduplicates",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/markduplicates/sample_id_lanes_1234_trim_star_sorted_merged_md.cram",
        "tags": ["cram", "mip-rna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("markduplicates",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/markduplicates/sample_id_lanes_1234_trim_star_sorted_merged_md.cram.crai",
        "tags": ["cram-index", "mip-rna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("gatk_asereadcounter",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/gatk_asereadcounter/sample_id_lanes_1234_trim_star_sorted_merged_md_splitncigar_brecal_haptc_filtered_ase.csv",
        "tags": ["gatk-asereadcounter", "mip-rna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("bootstrapann",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/sample_id/bootstrapann/sample_id_lanes_1234_trim_star_sorted_merged_md_splitncigar_brecal_haptc_filtered_ase_bootstr.vcf",
        "tags": ["bootstrapann", "mip-rna", "sample_id"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("bcftools_merge",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/bcftools_merge/case_id_comb.vcf",
        "tags": ["bcftools-merge", "case_id", "mip-rna"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("varianteffectpredictor",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/varianteffectpredictor/case_id_comb_vep.vcf",
        "tags": ["case_id", "mip-rna", "varianteffectpredictor"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("version_collect_ar",),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/version_collect_ar/case_id_vcol.yaml",
        "tags": ["case_id", "mip-rna", "version-collect-ar"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("multiqc_ar", "json"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/multiqc_ar/multiqc_data/multiqc_data.json",
        "tags": ["case_id", "mip-rna", "multiqc-json"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("multiqc_ar", "html"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/multiqc_ar/multiqc_report.html",
        "tags": ["case_id", "mip-rna", "multiqc-html"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "sample_info"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id_qc_sample_info.yaml",
        "tags": ["case_id", "mip-analyse", "mip-rna", "sample-info"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "references_info"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/reference_info.yaml",
        "tags": ["case_id", "mip-analyse", "mip-rna", "reference-info"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "log"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/mip_log/2020-02-28/mip_2020-02-28T10:22:26.log",
        "tags": ["case_id", "log", "mip-analyse", "mip-rna"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "config_analysis"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id_config.yaml",
        "tags": ["case_id", "config-analysis", "mip-analyse", "mip-rna"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "pedigree"),
        "path": "/path/to/rare-disease/cases/case_id/pedigree.yaml",
        "tags": ["case_id", "mip-analyse", "mip-rna", "pedigree"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "config"),
        "path": "/path/to/servers/config/cluster.ourplace.se/mip8.2-rna-stage.yaml",
        "tags": ["case_id", "config", "mip-analyse", "mip-rna"],
    },
    {
        "archive": False,
        "deliverables_tag_map": ("mip_analyse", "pedigree_fam"),
        "path": "/path/to/rare-disease/cases/case_id/analysis/case_id/case_id.fam",
        "tags": ["case_id", "mip-analyse", "mip-rna", "pedigree-fam"],
    },
]



"""Test MIP RNA files"""

from snapshottest import Snapshot

from cg.apps.mip import parse_sampleinfo


def test_parse_sampleinfo_rna_result_contents(snapshot: Snapshot, files_raw: dict):
    """test parse_sampleinfo_rna using snapshot

    Note: to retake all snapshots run `pytest --snapshot-update`

    Args:
        snapshot (Snapshot): a file with a snapshot of the correct test result, see
        cg/cg/tests/snapshots/snap_test_apps_mip_files.py
        files_raw (dict): dict of raw .yaml files
    """
    # GIVEN an RNA sample info file
    sampleinfo_raw = files_raw["rna_sampleinfo"]

    # WHEN parsing the file
    sampleinfo_data = parse_sampleinfo.parse_sampleinfo_rna(sampleinfo_raw)

    # THEN the result should contains the data as specified by the MIP team
    snapshot.assert_match(sampleinfo_data)



from pathlib import Path

import pytest

from cg.apps.housekeeper.models import InputBundle
from cg.constants import Pipeline
from cg.meta.workflow.fluffy import FluffyAnalysisAPI
from tests.mocks.process_mock import ProcessMock
import datetime as dt


@pytest.fixture(scope="function")
def fluffy_case_id_existing():
    return "lovedkitten"


@pytest.fixture(scope="function")
def fluffy_case_id_non_existing():
    return "nakedmolerat"


@pytest.fixture(scope="function")
def fluffy_sample_lims_id():
    return "ACC9001A1"


@pytest.fixture(scope="function")
def fluffy_dir(tmpdir_factory):
    return tmpdir_factory.mktemp("fluffy")


@pytest.fixture(scope="function")
def fluffy_cases_dir(tmpdir_factory, fluffy_dir):
    return tmpdir_factory.mktemp("cases")


@pytest.fixture(scope="function")
def fluffy_success_output_summary(tmpdir_factory):
    output_dir = tmpdir_factory.mktemp("output")
    file_path = Path(output_dir, "summary.csv")
    file_path.touch(exist_ok=True)
    return file_path


@pytest.fixture(scope="function")
def fluffy_success_output_multiqc(tmpdir_factory):
    output_dir = tmpdir_factory.mktemp("output")
    file_path = Path(output_dir, "multiqc_report.html")
    file_path.touch(exist_ok=True)
    return file_path


@pytest.fixture(scope="function")
def fluffy_success_output_aberrations(tmpdir_factory):
    output_dir = tmpdir_factory.mktemp("output")
    file_path = Path(output_dir, "WCXpredict_aberrations.filt.bed")
    file_path.touch(exist_ok=True)
    return file_path


@pytest.fixture(scope="function")
def samplesheet_fixture_path():
    return Path("tests/fixtures/apps/fluffy/SampleSheet.csv")


@pytest.fixture(scope="function")
def fastq_file_fixture_path():
    return Path("tests/fixtures/apps/fluffy/fluffy_fastq.fastq.gz")


@pytest.fixture(scope="function")
def deliverables_yaml_fixture_path():
    return Path("tests/fixtures/apps/fluffy/deliverables.yaml")


@pytest.fixture()
def fluffy_hermes_deliverables_response_data(
    fluffy_case_id_existing,
    fluffy_sample_lims_id,
    fluffy_success_output_multiqc,
    fluffy_success_output_summary,
    fluffy_success_output_aberrations,
):
    return InputBundle(
        **{
            "files": [
                {
                    "path": fluffy_success_output_summary.as_posix(),
                    "tags": ["metrics", fluffy_case_id_existing, "nipt"],
                },
                {
                    "path": fluffy_success_output_multiqc.as_posix(),
                    "tags": ["multiqc-html", fluffy_case_id_existing, "nipt"],
                },
                {
                    "path": fluffy_success_output_aberrations.as_posix(),
                    "tags": ["wisecondor", "cnv", fluffy_sample_lims_id, "nipt"],
                },
            ],
            "created": dt.datetime.now(),
            "name": fluffy_case_id_existing,
        }
    )


@pytest.fixture(scope="function")
def fluffy_fastq_hk_bundle_data(fastq_file_fixture_path, fluffy_sample_lims_id) -> dict:
    return {
        "name": fluffy_sample_lims_id,
        "created": dt.datetime.now(),
        "version": "1.0",
        "files": [
            {"path": fastq_file_fixture_path.as_posix(), "tags": ["fastq"], "archive": False}
        ],
    }


@pytest.fixture(scope="function")
def fluffy_samplesheet_bundle_data(samplesheet_fixture_path) -> dict:
    return {
        "name": "flowcell",
        "created": dt.datetime.now(),
        "version": "1.0",
        "files": [
            {"path": samplesheet_fixture_path.as_posix(), "tags": ["samplesheet"], "archive": False}
        ],
    }


@pytest.fixture(scope="function")
def fluffy_populated_status_db(base_store, helpers, fluffy_case_id_existing, fluffy_sample_lims_id):
    example_fluffy_case = helpers.add_case(
        base_store,
        internal_id=fluffy_case_id_existing,
        case_id=fluffy_case_id_existing,
        data_analysis=Pipeline.FLUFFY,
    )
    example_fluffy_sample = helpers.add_sample(
        base_store,
        internal_id=fluffy_sample_lims_id,
        is_tumour=False,
        application_type="tgs",
        reads=100,
        sequenced_at=dt.datetime.now(),
    )
    helpers.add_flowcell(base_store, flowcell_id="flowcell", samples=[example_fluffy_sample])
    helpers.add_relationship(base_store, case=example_fluffy_case, sample=example_fluffy_sample)
    return base_store


@pytest.fixture(scope="function")
def fluffy_populated_housekeeper_store(
    real_housekeeper_api, helpers, fluffy_fastq_hk_bundle_data, fluffy_samplesheet_bundle_data
):
    helpers.ensure_hk_bundle(
        store=real_housekeeper_api,
        bundle_data=fluffy_fastq_hk_bundle_data,
    )
    helpers.ensure_hk_bundle(store=real_housekeeper_api, bundle_data=fluffy_samplesheet_bundle_data)
    return real_housekeeper_api


@pytest.fixture(scope="function")
def fluffy_context(
    fluffy_populated_housekeeper_store,
    trailblazer_api,
    hermes_api,
    fluffy_populated_status_db,
    fluffy_cases_dir,
    lims_api,
) -> dict:
    fluffy_analysis_api = FluffyAnalysisAPI(
        housekeeper_api=fluffy_populated_housekeeper_store,
        trailblazer_api=trailblazer_api,
        hermes_api=hermes_api,
        lims_api=lims_api,
        status_db=fluffy_populated_status_db,
        config={
            "root_dir": fluffy_cases_dir,
            "binary_path": "echo",
            "config_path": "/dev/null/config.json",
        },
    )
    fluffy_analysis_api.process = ProcessMock()
    return {"fluffy_analysis_api": fluffy_analysis_api}



"""Fixtures for the housekeeper tests"""

import datetime

import pytest
from housekeeper.store import models

from cg.apps.housekeeper.hk import HousekeeperAPI


@pytest.fixture(name="hk_config")
def fixture_hk_config(root_path) -> dict:
    """Return a dictionary with housekeeper api configs for testing"""
    config = {"housekeeper": {"database": "sqlite:///:memory:", "root": str(root_path)}}
    return config


@pytest.fixture(scope="function", name="housekeeper_api")
def fixture_housekeeper_api(hk_config):
    """Setup Housekeeper store."""
    _api = HousekeeperAPI(hk_config)
    _api.initialise_db()
    yield _api
    _api.destroy_db()


@pytest.fixture(scope="function", name="populated_housekeeper_api")
def fixture_populated_housekeeper_api(housekeeper_api, bundle_data):
    """Setup Housekeeper store."""
    _api = housekeeper_api
    bundle_obj, version_obj = _api.add_bundle(bundle_data)
    _api.add_commit(bundle_obj, version_obj)
    return _api


@pytest.fixture(name="a_date")
def fixture_a_date() -> datetime.datetime:
    """Return a datetime object with a date"""

    return datetime.datetime(2020, 5, 1)


@pytest.fixture(name="later_date")
def fixture_later_date() -> datetime.datetime:
    """Return a datetime object with a date later than a_date"""

    return datetime.datetime(2020, 5, 3)


@pytest.fixture(name="minimal_bundle_obj")
def fixture_minimal_bundle_obj(a_date, case_id, housekeeper_api) -> models.Bundle:
    """Return a bundle object with minimal information (name and created_at)"""
    _bundle_obj = housekeeper_api.new_bundle(name=case_id, created_at=a_date)

    return _bundle_obj


@pytest.fixture(name="tags")
def fixture_tags() -> list:
    """Return a list of housekeeper tags"""
    return ["bed"]


@pytest.fixture(name="bundle_data")
def fixture_bundle_data(a_date, case_id, bed_file, tags) -> dict:
    """Return a dictionary with bundle info in the correct format"""
    _bundle_data = {
        "name": case_id,
        "created": a_date,
        "files": [{"path": bed_file, "archive": False, "tags": tags}],
    }

    return _bundle_data



"""Fixtures for the shipping api tests"""

from typing import Dict
from pathlib import Path
import pytest

from cg.apps.shipping import ShippingAPI


@pytest.fixture(name="binary_path")
def fixture_binary_path() -> Path:
    return Path("path/to/shipping")


@pytest.fixture(name="host_config")
def fixture_host_config() -> Path:
    return Path("path/to/host_config.yml")


@pytest.fixture(name="shipping_configs")
def fixture_shipping_configs(binary_path: Path, host_config: Path) -> Dict[str, str]:
    return dict(host_config=str(host_config), binary_path=str(binary_path))


@pytest.fixture(name="shipping_api")
def fixture_shipping_api(shipping_configs: Dict[str, str]) -> ShippingAPI:
    api = ShippingAPI(config=shipping_configs)
    api.set_dry_run(dry_run=True)
    return api



"""Schemas for scout serialisation"""

from typing import List, Optional
from typing_extensions import Literal
from pydantic import BaseModel, Field, validator
from datetime import datetime


class Individual(BaseModel):
    bam_file: Optional[str] = None
    individual_id: str
    sex: Literal["0", "1", "2", "other"]
    father: Optional[str]
    mother: Optional[str]
    phenotype: Literal[1, 2, 0]
    analysis_type: str = "wgs"

    @validator("father", "mother")
    def convert_to_zero(cls, v):
        if v is None:
            return "0"
        return v

    @validator("sex")
    def convert_sex_to_zero(cls, v):
        if v == "other":
            return "0"
        return v


class Panel(BaseModel):
    panel_name: str


class Phenotype(BaseModel):
    phenotype_id: str
    feature: str


class Gene(BaseModel):
    hgnc_id: int
    hgnc_symbol: Optional[str]
    region_annotation: Optional[str]
    functional_annotation: Optional[str]
    sift_prediction: Optional[str]
    polyphen_prediction: Optional[str]


class ScoutExportCase(BaseModel):
    id: str = Field(str, alias="_id")
    analysis_date: datetime
    owner: str
    causatives: Optional[List[str]] = None
    collaborators: List[str] = []
    individuals: List[Individual]
    genome_build: Optional[str]
    panels: Optional[List[Panel]]
    rank_model_version: Optional[str]
    sv_rank_model_version: Optional[str]
    rank_score_threshold: int = 5
    phenotype_terms: Optional[List[Phenotype]]
    phenotype_groups: Optional[List[Phenotype]]
    diagnosis_phenotypes: Optional[List[int]]
    diagnosis_genes: Optional[List[int]]

    @validator("genome_build")
    def convert_genome_build(cls, v):
        if v is None:
            return "37"
        return v


class Genotype(BaseModel):
    genotype_call: str
    allele_depths: List[int]
    read_depth: int
    genotype_quality: int
    sample_id: str


class Variant(BaseModel):
    document_id: str
    variant_id: str
    chromosome: str
    position: int
    dbsnp_id: Optional[str] = None
    reference: str
    alternative: str
    quality: Optional[float] = None
    filters: Optional[List[str]] = None
    end: int
    rank_score: int
    category: str
    sub_category: str
    genes: List[Gene]
    samples: List[Genotype]
